<?php
/**
* LICENCIA
*
* Este programa se propociona "tal cual", sin garantía de ningún tipo más allá del soporte
* pactado a la hora de adquirir el programa.
*
* En ningún caso los autores o titulares del copyright serán responsables de ninguna
* reclamación, daños u otras responsabilidades, ya sea en un litigio, agravio o de otro
* modo, que surja de o en conexión con el programa o el uso u otro tipo de acciones
* realizadas con el programa.
*
* Este programa no puede modificarse ni distribuirse sin el consentimiento expreso del autor.
*
*    @author    Carlos Fillol Sendra <festeweb@festeweb.com>
*    @copyright 2014 Fes-te web! - www.festeweb.com
*    @license   http://www.festeweb.com/static/licenses/fs2psinvoices_1.1.0.txt
*/

if (!defined('_CAN_LOAD_FILES_')) exit;

include_once(_PS_MODULE_DIR_.'fs2ps/Fs2psTools.php');

class fs2psinvoices extends Module
{
	const MIN_VER = '1.5.0.0';
	const MAX_VER = '1.7.999.999';
	
	public function __construct()
	{
		$this->name = 'fs2psinvoices';
		$this->tab = 'migration_tools';
		$this->version = '2.8.14';
		$this->author = 'Fes-te web!';
		$this->need_instance = 0;
		$this->bootstrap = true;
		
		$this->ps_versions_compliancy = array('min' => self::MIN_VER, 'max' => self::MAX_VER);
		if (version_compare(_PS_VERSION_, self::MIN_VER) < 0 ||
			version_compare(_PS_VERSION_, self::MAX_VER) > 0)
		{
			// Avoid problems with PS before 1.5
			$this->_errors[] = $this->l('The version of your module is not compliant with your PrestaShop version.');
			return false;
		}
		
		parent::__construct();

		$this->displayName = 'fs2psinvoices';
		$this->description = 'Módulo para compartir con los clientes las facturas generadas en FactuSOL a través de Prestashop';
		$this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

		if (!Configuration::get('FS2PS_KEY'))
			$this->warning = '
				Debes tener instalado y configurado el módulo de FS2PS para que FS2PS INVOICES funcione.
			';
	}

	public function install()
	{
	    // Este modulo depende del módulo fs2ps
	    if (!file_exists(_PS_MODULE_DIR_.'fs2ps') || 
	        !parent::install() || 
	        !$this->registerHook('orderDetailDisplayed')) {
	       return false;
	    } 
	    
		$upgrate_versions = array('2.8.1');
		foreach ($upgrate_versions as $ver)
		{
			include(dirname(__FILE__).'/upgrade/install-'.$ver.'.php');
			$upgrade_func = 'upgrade_module_'.str_replace('.', '_', $ver);
			
			if (!$upgrade_func($this))
				return false;
		}
		
		$this->createAdminTabs();

		return true;
	}

	public function uninstall()
	{
		if (!parent::uninstall() || !$this->unregisterHook('orderDetailDisplayed'))
			return false;

		$this->removeAdminTabs();
			
		return true;
	}

	protected function createAdminTabs()
	{
	    /* If the "AdminFs2psInvoicesOrder" tab does not exist yet, create it */
	    if (!$id_tab = Tab::getIdFromClassName('AdminFs2psInvoicesOrder')) {
	        $tab = new Tab();
	        $tab->class_name = 'AdminFs2psInvoicesOrder';
	        $tab->module = $this->name;
	        $tab->id_parent = version_compare(_PS_VERSION_, '1.7.0.0') >= 0? (int)Tab::getIdFromClassName('AdminTools') : 0;
	        $tab->name = Fs2psTools::multiLangField('Facturas FS2PS', true);
	        if (!$tab->save()) {
	            $this->_errors[] = $this->trans('Unable to create the "AdminFs2psInvoicesOrder" tab', array(), 'Modules.Fs2psInvoices.Admin');
	        }
	        /*
	        if (!@copy(dirname(__FILE__) . DIRECTORY_SEPARATOR . 'logo.gif', _PS_ROOT_DIR_ . DIRECTORY_SEPARATOR . 'img' . DIRECTORY_SEPARATOR . 't' . DIRECTORY_SEPARATOR . 'AdminFs2psInvoicesOrder.gif')) {
	            return $this->_abortInstall($this->trans('Unable to copy logo.gif in %s', array(_PS_ROOT_DIR_ . DIRECTORY_SEPARATOR . 'img' . DIRECTORY_SEPARATOR . 't' . DIRECTORY_SEPARATOR), 'Modules.Fs2psInvoices.Admin'));
	        }
	        */
	    } else {
	        $tab = new Tab((int) $id_tab);
	    }
	}
	
	protected function removeAdminTabs()
	{
	    if ($id_tab = Tab::getIdFromClassName('AdminFs2psInvoicesOrder')) {
	        $tab = new Tab($id_tab);
	        $tab->delete();
	    }
	}
	
	public function hookOrderDetailDisplayed($params)
	{
	    $id_order = intval($params['order']->id);
	    $invoices = Fs2psTools::dbSelect('
            select i.id, i.num, i.name
            from @DB_fs2ps_order_invoice oi
            inner join @DB_fs2ps_invoice i on i.id=oi.invoice
            where oi.order='.$id_order.'
        ');
	    $this->context->smarty->assign(array('invoices' => $invoices));
	    return $this->display(__FILE__, 'views/templates/hook/orderdetaildisplayed.tpl');
	    
	}
		
}
